{% extends "!layout.html" %}

{%- block rootrellink %}
    <li><a href="http://saltstack.org">&laquo; SaltStack.org</a>&nbsp;|&nbsp;</li>
    <li><a href="{{ pathto('index') }}">Documentation home</a></li>
{%- endblock %}
