{% if networking %}NETWORKING={{networking}}
{%endif%}{% if hostname %}HOSTNAME={{hostname}}
{%endif%}{% if gateway %}GATEWAY={{gateway}}
{%endif%}{% if gatewaydev %}GATEWAYDEV={{gatewaydev}}
{%endif%}{% if nisdomain %}NISDOMAIN={{nisdomain}}
{%endif%}{% if nozeroconf %}NOZEROCONF={{nozeroconf}}{%endif%}
